@@ -7,5 +7,6 @@
   cat("   ##########################\n\n")
   
   cat(" - to start, type '?adegenet'\n")
-  cat(" - to browse adegenet website, type 'adegenetWeb()'\n\n")
+  cat(" - to browse adegenet website, type 'adegenetWeb()'\n")
+  cat(" - to post questions/comments: adegenet-forum@lists.r-forge.r-project.org\n\n")
 }
